// DictionaryEditorView.cpp : implementation of the CDictionaryEditorView class
//

#include "stdafx.h"
#include "DictionaryEditor.h"
#include "SmartListView.h"
#include "DictionaryEditorDoc.h"
#include "DictionaryEditorView.h"
#include "AddWordDlg.h"
#include "MainFrm.h"
#include "ProgressBar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDictionaryEditorView

IMPLEMENT_DYNCREATE(CDictionaryEditorView, CListView)

BEGIN_MESSAGE_MAP(CDictionaryEditorView, CListView)
	//{{AFX_MSG_MAP(CDictionaryEditorView)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_COMMAND(ID_WORDS_ADD, OnWordsAdd)
	ON_COMMAND(ID_WORDS_DELETE, OnWordsDelete)
	ON_COMMAND(ID_WORDS_EDIT, OnWordsEdit)
	ON_UPDATE_COMMAND_UI(ID_WORDS_EDIT, OnUpdateWordsEdit)
	ON_UPDATE_COMMAND_UI(ID_WORDS_DELETE, OnUpdateWordsDelete)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	ON_COMMAND(ID_TOOLS_TRIM, OnToolsTrim)
	ON_COMMAND(ID_TOOLS_LOWERCASE, OnToolsLowercase)
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED, OnItemchanged)
	ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
	ON_COMMAND(ID_FILE_IMPORT, OnFileImport)
	ON_COMMAND(ID_TOOLS_FIND, OnToolsFind)
	ON_COMMAND(ID_TOOLS_FINDNEXT, OnToolsFindnext)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDictionaryEditorView construction/destruction

CDictionaryEditorView::CDictionaryEditorView()
{
	// TODO: add construction code here

}

CDictionaryEditorView::~CDictionaryEditorView()
{
}

BOOL CDictionaryEditorView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= LVS_REPORT | LVS_EX_REGIONAL;
	cs.style ^= LVS_EDITLABELS;

	return CListView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDictionaryEditorView drawing

void CDictionaryEditorView::OnDraw(CDC* pDC)
{
	CDictionaryEditorDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

void CDictionaryEditorView::OnInitialUpdate()
{
	CListCtrl& m_List = GetListCtrl();

	DWORD dwStyle = LVS_EX_FULLROWSELECT;

	dwStyle |= LVS_EX_GRIDLINES;

	m_List.SetExtendedStyle(dwStyle);

	while(m_List.DeleteColumn(0));

	m_List.InsertColumn(0, "Word", NULL, 250, NULL);

	CListView::OnInitialUpdate();
}

/////////////////////////////////////////////////////////////////////////////
// CDictionaryEditorView diagnostics

#ifdef _DEBUG
void CDictionaryEditorView::AssertValid() const
{
	CListView::AssertValid();
}

void CDictionaryEditorView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CDictionaryEditorDoc* CDictionaryEditorView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDictionaryEditorDoc)));
	return (CDictionaryEditorDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDictionaryEditorView message handlers

void CDictionaryEditorView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CListCtrl& m_List = GetListCtrl();

	SetRedraw(FALSE);

	m_List.DeleteAllItems();

	CWaitCursor wait;
	CDictionaryEditorDoc *pDoc = GetDocument();
	int nNum = pDoc->NumWords();

	int x;
	for(x=0; x < nNum; x++)
	{
		CString strWord;

		strWord = pDoc->GetWord(x);

		int nItem = m_List.InsertItem(x, strWord, 0);
		
	}

	m_List.SetRedraw(TRUE);

	CString strStatus;
	
	int nSelected;

	nSelected = m_List.GetSelectedCount();
	
	if (nSelected != LB_ERR)
	{
		CString strStat1;
		strStat1 = FormatString(nSelected, ",");

		CString strStat2;
		strStat2 = FormatString(GetDocument()->array.GetSize(), ",");

		strStatus = strStat1;
		strStatus += " of ";
		strStatus += strStat2;
	}
	else
	{
		strStatus = "";
	}

	((CMainFrame*)AfxGetMainWnd())->SetPaneText(1, strStatus);

}


void CDictionaryEditorView::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	CWaitCursor wait;
	CListCtrl& m_List = GetListCtrl();

	int x = m_List.GetItemCount();

	if (x < 2)
	{
		return;
	}

	SortTextItems(pNMListView->iSubItem, TRUE, 0, -1);

	GetDocument()->array.RemoveAll();

	for (int j = 0; j < x; j++)
	{
		CString strWord;

		strWord = m_List.GetItemText(j, 0);

//		m_temp = new CDictionary(strWord);

		GetDocument()->AddWord(strWord);
	}

	GetDocument()->SetModifiedFlag();

	*pResult = 0;
}

void CDictionaryEditorView::OnWordsAdd() 
{
	CListCtrl& m_List = GetListCtrl();

	CAddWordDlg dlg;
	dlg.Option1 = 1;
	if (dlg.DoModal() == IDOK)
	{
		int x = GetDocument()->NumWords();

		BOOL bFound;
		bFound = FALSE;
		CString strWord1 = dlg.m_strWord;

		strWord1.MakeLower();

		CString strWord2;

		for (int j = 0; j < x; j++)
		{
			strWord2 = GetDocument()->GetWord(j);

			strWord2.MakeLower();

			if (strWord1 == strWord2)
			{
				bFound = TRUE;
				break;
			}
		}

		if (bFound == TRUE)
		{
			return;
		}
		
		GetDocument()->AddWord(dlg.m_strWord);
		GetDocument()->SetModifiedFlag();
		
		int nItem = m_List.InsertItem(m_List.GetItemCount(), dlg.m_strWord, 0);		
	}

	CString strStatus;
	
	int nSelected;

	nSelected = m_List.GetSelectedCount();
	
	if (nSelected != LB_ERR)
	{
		CString strStat1;
		strStat1 = FormatString(nSelected, ",");

		CString strStat2;
		strStat2 = FormatString(GetDocument()->array.GetSize(), ",");

		strStatus = strStat1;
		strStatus += " of ";
		strStatus += strStat2;
	}
	else
	{
		strStatus = "";
	}

	((CMainFrame*)AfxGetMainWnd())->SetPaneText(1, strStatus);
}

void CDictionaryEditorView::OnWordsDelete() // Delete all the selected words
{
	CWaitCursor wait;

	CListCtrl& m_List = GetListCtrl();

	int nSel = m_List.GetSelectedCount();

	m_List.SetRedraw(FALSE);

	int i = m_List.GetNextItem( -1, LVNI_ALL | LVNI_SELECTED);
	while( i != -1 )
	{
		m_List.DeleteItem(i);
		GetDocument()->array.RemoveAt(i, 1);
		i = m_List.GetNextItem( -1, LVNI_ALL | LVNI_SELECTED);
	}

	m_List.SetRedraw(TRUE);

	GetDocument()->SetModifiedFlag();

	CString strStatus;
	
	int nSelected;

	nSelected = m_List.GetSelectedCount();
	
	if (nSelected != LB_ERR)
	{
		CString strStat1;
		strStat1 = FormatString(nSelected, ",");

		CString strStat2;
		strStat2 = FormatString(GetDocument()->array.GetSize(), ",");

		strStatus = strStat1;
		strStatus += " of ";
		strStatus += strStat2;
	}
	else
	{
		strStatus = "";
	}

	((CMainFrame*)AfxGetMainWnd())->SetPaneText(1, strStatus);
}

void CDictionaryEditorView::OnWordsEdit() // Edit the currently selected word
{
	CListCtrl& m_List = GetListCtrl();

	int nSel = m_List.GetNextItem( -1, LVNI_ALL | LVNI_SELECTED);

	CAddWordDlg dlg;

	dlg.Option1 = 2;

	if (nSel == -1)
	{
		return;
	}

	CString strWord;

	strWord = GetDocument()->GetWord(nSel);

	dlg.m_strWord = strWord;

	if (dlg.DoModal() == IDOK)
	{
		if (dlg.m_strWord != strWord)
		{
			GetDocument()->array.SetAt(nSel, dlg.m_strWord);

			m_List.SetItemText(nSel, 0, dlg.m_strWord);

			GetDocument()->SetModifiedFlag();
		}
	}

	CString strStatus;
	
	int nSelected;

	nSelected = m_List.GetSelectedCount();
	
	if (nSelected != LB_ERR)
	{
		CString strStat1;
		strStat1 = FormatString(nSelected, ",");

		CString strStat2;
		strStat2 = FormatString(GetDocument()->array.GetSize(), ",");

		strStatus = strStat1;
		strStatus += " of ";
		strStatus += strStat2;
	}
	else
	{
		strStatus = "";
	}

	((CMainFrame*)AfxGetMainWnd())->SetPaneText(1, strStatus);
}

void CDictionaryEditorView::OnUpdateWordsEdit(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CDictionaryEditorView::OnUpdateWordsDelete(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CDictionaryEditorView::OnDblclk(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnWordsEdit();	
	*pResult = 0;
}

void CDictionaryEditorView::OnToolsTrim() 
{
	CWaitCursor wait; // Display a wait cursor

	CListCtrl& m_List = GetListCtrl(); // Get handle to the base List Control

	int x = m_List.GetItemCount(); // Get the number of words in the list control
	
	GetDocument()->array.RemoveAll(); // Remove all words in our dictionary

	for (int j = 0; j < x; j++) // For each item in the list
	{
		CString strWord;

		strWord = m_List.GetItemText(j, 0); // Get the word

		strWord.TrimLeft(); // Remove leading spaces
		strWord.TrimRight(); // Remove trailing spaces

		m_List.SetItemText(j, 0, strWord); // Overwrite the old word with the new trimmed word

		GetDocument()->AddWord(strWord); // Add the word to the dictionary
	}

	GetDocument()->SetModifiedFlag(); // Set the document to be modified
}

void CDictionaryEditorView::OnToolsLowercase() 
{
	CWaitCursor wait; // Display a wait cursor

	CListCtrl& m_List = GetListCtrl(); // Get handle to the base List Control

	int x = m_List.GetItemCount(); // Get the number of words in the list control

	GetDocument()->array.RemoveAll(); // Remove all words in our dictionary

	for (int j = 0; j < x; j++) // For each item in the list
	{
		CString strWord;

		strWord = m_List.GetItemText(j, 0); // Get the word

		strWord.MakeLower(); // Make the word lowercase

		m_List.SetItemText(j, 0, strWord); // Overwrite the old word with the new lowercase word

		GetDocument()->AddWord(strWord); // Add the word to the dictionary
	}

	GetDocument()->SetModifiedFlag(); // Set the document to be modified
}

CString CDictionaryEditorView::FormatString(UINT nVal, CString strSymbol) // Format a number so that it has "," for each 3 characters to make it easier to read
{
	CString strNumber;
	strNumber.Format(("%u"), nVal);

	int nLength = strNumber.GetLength();
	int nCount = (nLength - 1) / 3;

	if (!nCount)
		return strNumber;

	for (int i=1; i<=nCount; i++)
	{
		strNumber = strNumber.Left(nLength - (i * 3)) + strSymbol + strNumber.Right((i * 3) + (i - 1));
	}
	return strNumber;
}

void CDictionaryEditorView::OnItemchanged(NMHDR* pNMHDR, LRESULT* pResult) // The item's changed, so update the selected item display in the status bar
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	CListCtrl& m_List = GetListCtrl();
	
	CString strStatus;
	
	int nSelected;

	nSelected = m_List.GetSelectedCount();
	
	if (nSelected != LB_ERR)
	{
		CString strStat1;
		strStat1 = FormatString(nSelected, ",");

		CString strStat2;
		strStat2 = FormatString(GetDocument()->array.GetSize(), ",");

		strStatus = strStat1;
		strStatus += " of ";
		strStatus += strStat2;
	}
	else
	{
		strStatus = "";
	}

	((CMainFrame*)AfxGetMainWnd())->SetPaneText(1, strStatus);

	*pResult = 0;
}

void CDictionaryEditorView::OnFileExport() // Export all the selected words if any are selected, otherwise export all words
{
	CFileDialog	dlg (FALSE, ("txt"), "", OFN_OVERWRITEPROMPT,
						 ("Text Files (*.txt)|*.txt|All Files (*.*)|*.*||"),
						 NULL);	
	dlg.m_ofn.lpstrTitle = _T("Export words");

	CString strPath;
	if (dlg.DoModal() == IDOK )
	{
		strPath = dlg.GetPathName();

		::DeleteFile((LPCTSTR)strPath);

		CWaitCursor wait;
		CFile cfSettingsFile ( strPath, CFile::modeCreate | CFile::modeNoTruncate | CFile::modeReadWrite );
		CArchive ar ( &cfSettingsFile, CArchive::store );

		CListCtrl& m_List = GetListCtrl();

		int nSel = m_List.GetSelectedCount();

		if (nSel > 0 )
		{
			CProgressBar Bar("Exporting words ", 40, nSel);
			Bar.SetPos(0);
			int k = 0;
			int i = m_List.GetNextItem( -1, LVNI_ALL | LVNI_SELECTED);
			while( i != -1 )
			{
				m_List.SetItemState(i, ~LVIS_SELECTED, LVIS_SELECTED);

				Bar.SetPos(k);

				CString strWord;

				CString strLine = "";

				strWord = m_List.GetItemText(i, 0);

				strLine += strWord;
				
				ar.WriteString(strLine + "\r\n");

				k++;
					
				i = m_List.GetNextItem( -1, LVNI_ALL | LVNI_SELECTED);
			}
		}
		else
		{
			int x = m_List.GetItemCount();

			CProgressBar Bar("Exporting words ", 40, x);
			Bar.SetPos(0);
			for (int j = 0; j < x; j++)
			{
				Bar.SetPos(j);

				CString strWord;

				CString strLine = "";

				strWord = m_List.GetItemText(j, 0);

				strLine += strWord;
				
				ar.WriteString(strLine + "\r\n");
			}
		}
		((CMainFrame*)AfxGetMainWnd())->SetPaneText(0, "Ready");
	}	
}

void CDictionaryEditorView::OnFileImport() // Import words from a text file, reading the file line by line
{
	CListCtrl& m_List = GetListCtrl();

	CFileDialog	dlg (TRUE, ("txt"), "", OFN_FILEMUSTEXIST,
						 ("Text Files (*.txt)|*.txt|All Files (*.*)|*.*||"),
						 NULL);

	dlg.m_ofn.lpstrTitle = _T("Import words");

	CString strPath;
	if (dlg.DoModal() == IDOK )
	{
		strPath = dlg.GetPathName();

		((CMainFrame*)AfxGetMainWnd())->SetPaneText(0, "Importing words...");
		CWaitCursor wait;
		
		CFile cfSettingsFile ( strPath, CFile::modeNoTruncate | CFile::modeReadWrite );
		CArchive ar ( &cfSettingsFile, CArchive::load );

		CString strLine;
		if( !ar.ReadString( strLine ) )
			return;

		do
		{
			if( strLine.GetLength() == 0 )
				continue;

			CString strWord = "";

			CString strText = strLine;
			
			strWord = strText;

			strWord.TrimLeft();
			strWord.TrimRight();
			
			int x = GetDocument()->NumWords();

			BOOL bFound;
			bFound = FALSE;
			CString strWd1 = strWord;

			strWd1.MakeLower();

		/*	for (int j = 0; j < x; j++) // This is code that checks if a word exists already before adding it
				// Very slow!!!
			{
				CString strWd;

				GetDocument()->GetWord(j, strWd);
				
				CString strWd2 = strWd;
				strWd2.MakeLower();

				if (strWd1 == strWd2)
				{
					bFound = TRUE;

				}
			}

			if (bFound == FALSE)
			{*/
				GetDocument()->AddWord(strWord);

				int nItem = m_List.InsertItem(m_List.GetItemCount(), strWord, 0);
			//}

		}while( ar.ReadString( strLine ) );	

		((CMainFrame*)AfxGetMainWnd())->SetPaneText(0, "Ready");

		GetDocument()->SetModifiedFlag(TRUE);
	}
	
}

void CDictionaryEditorView::OnToolsFind() 
{
	CListCtrl& m_List = GetListCtrl(); // Get handle to the base List Control

	m_nFind = 0; // Reset FindNext counter to 0

	int x = m_List.GetItemCount(); // Get the number of words in the list control
	
	for (int j = m_nFind; j < x; j++) // For each item in the list
	{
		CString strWord;

		strWord = m_List.GetItemText(j, 0); // Get the word

		// Does that word have one of the following characters in it?
		if (strWord.FindOneOf("") != -1)
		{
			m_List.SetItemState(j, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED); // Select the word in the list
			m_List.EnsureVisible(j, FALSE); // Make sure it's visible in the list

			m_nFind = j + 1; // Update the FindNext counter

			return; // Return as we've found what we wanted
		}
	}
}

void CDictionaryEditorView::OnToolsFindnext() 
{
	CListCtrl& m_List = GetListCtrl();

	int x = m_List.GetItemCount();
	
	for (int j = m_nFind; j < x; j++)
	{
		CString strWord;

		strWord = m_List.GetItemText(j, 0);

		if (strWord.FindOneOf("") != -1)
		{
			m_List.SetItemState( j,LVIS_SELECTED | LVIS_FOCUSED , LVIS_SELECTED | LVIS_FOCUSED); 
			m_List.EnsureVisible( j,FALSE);

			m_nFind = j + 1;

			return;
		}
	}
}
